@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,7 +55,7 @@ import com.openexchange.mail.mime.MIMEMailException;
 
 /**
  * {@link SMTPException}
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class SMTPException extends MIMEMailException {
@@ -113,7 +113,16 @@ public final class SMTPException extends MIMEMailException {
         /**
          * No storage access because mail connection is not connected
          */
-        NOT_CONNECTED("No storage access because mail connection is not connected", Category.CODE_ERROR, 3012);
+        NOT_CONNECTED("No storage access because mail connection is not connected", Category.CODE_ERROR, 3012),
+        /**
+         * Unable to parse SMTP server URI "%1$s".
+         */
+        URI_PARSE_FAILED("Unable to parse SMTP server URI \"%1$s\".", Category.SETUP_ERROR, 3013),
+        /**
+         * The following recipient is not allowed: %1$s. Please remove associated address and try again.
+         */
+        RECIPIENT_NOT_ALLOWED("The following recipient is not allowed: %1$s. Please remove associated address and try again.", Category.USER_INPUT, 3014),
+        ;
 
         private final String message;
 
@@ -148,7 +157,7 @@ public final class SMTPException extends MIMEMailException {
 
     /**
      * Initializes a new {@link SMTPException}
-     * 
+     *
      * @param cause The cause
      */
     public SMTPException(final AbstractOXException cause) {
@@ -157,7 +166,7 @@ public final class SMTPException extends MIMEMailException {
 
     /**
      * Initializes a new {@link SMTPException}
-     * 
+     *
      * @param code The code
      * @param messageArgs The message arguments
      */
@@ -167,19 +176,19 @@ public final class SMTPException extends MIMEMailException {
 
     /**
      * Initializes a new {@link SMTPException}
-     * 
+     *
      * @param code The code
      * @param cause The cause
      * @param messageArgs The message arguments
      */
     public SMTPException(final Code code, final Throwable cause, final Object... messageArgs) {
-        super(SMTPProvider.PROTOCOL_SMTP, code.category, code.detailNumber, code.message, cause);
+        super(SMTPProvider.PROTOCOL_SMTP, code.getCategory(), code.getNumber(), code.getMessage(), cause);
         super.setMessageArgs(messageArgs);
     }
 
     /**
      * Initializes a new {@link SMTPException}
-     * 
+     *
      * @param code The code
      */
     public SMTPException(final Code code) {
