@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,7 +51,7 @@ package com.openexchange.smtp;
 
 import java.util.Map;
 import com.openexchange.groupware.contexts.Context;
-import com.openexchange.groupware.upload.impl.UploadFile;
+import com.openexchange.groupware.upload.UploadFile;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.Protocol;
 import com.openexchange.mail.api.AbstractProtocolProperties;
@@ -76,7 +76,7 @@ import com.openexchange.smtp.dataobjects.SMTPReferencedPart;
 
 /**
  * {@link SMTPProvider}
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class SMTPProvider extends TransportProvider {
@@ -90,7 +90,7 @@ public final class SMTPProvider extends TransportProvider {
 
     /**
      * Gets the singleton instance of SMTP provider
-     * 
+     *
      * @return The singleton instance of SMTP provider
      */
     public static SMTPProvider getInstance() {
@@ -120,7 +120,7 @@ public final class SMTPProvider extends TransportProvider {
     }
 
     @Override
-    public InfostoreDocumentMailPart getNewDocumentPart(final int documentId, final Session session) throws MailException {
+    public InfostoreDocumentMailPart getNewDocumentPart(final String documentId, final Session session) throws MailException {
         return new SMTPDocumentPart(documentId, session);
     }
 
