@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,35 +50,47 @@
 package com.openexchange.smtp;
 
 import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
+import static com.openexchange.mail.mime.converters.MIMEMessageConverter.saveChanges;
 import static com.openexchange.mail.mime.utils.MIMEMessageUtility.parseAddressList;
 import static com.openexchange.mail.text.TextProcessing.performLineFolding;
 import static java.util.regex.Matcher.quoteReplacement;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
+import java.net.InetAddress;
+import java.net.InetSocketAddress;
+import java.net.UnknownHostException;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
 import java.text.DateFormat;
 import java.util.Date;
 import java.util.Locale;
+import java.util.Map;
 import java.util.Properties;
 import java.util.Queue;
 import java.util.concurrent.ConcurrentLinkedQueue;
-import javax.activation.DataHandler;
 import javax.mail.Address;
 import javax.mail.Message.RecipientType;
 import javax.mail.MessagingException;
 import javax.mail.Multipart;
 import javax.mail.NoSuchProviderException;
-import javax.mail.Part;
 import javax.mail.Transport;
+import javax.mail.internet.IDNA;
 import javax.mail.internet.InternetAddress;
 import javax.mail.internet.MailDateFormat;
 import javax.mail.internet.MimeBodyPart;
 import javax.mail.internet.MimeMultipart;
+import javax.security.auth.Subject;
+import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Filter;
+import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.groupware.contexts.impl.ContextStorage;
 import com.openexchange.groupware.i18n.MailStrings;
 import com.openexchange.groupware.ldap.UserStorage;
+import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.i18n.tools.StringHelper;
+import com.openexchange.log.LogProperties;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.dataobjects.MailMessage;
@@ -87,6 +99,7 @@ import com.openexchange.mail.dataobjects.compose.ComposedMailMessage;
 import com.openexchange.mail.mime.ContentType;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.MessageHeaders;
+import com.openexchange.mail.mime.MimeHeaderNameChecker;
 import com.openexchange.mail.mime.converters.MIMEMessageConverter;
 import com.openexchange.mail.mime.utils.MIMEMessageUtility;
 import com.openexchange.mail.transport.MailTransport;
@@ -103,24 +116,61 @@ import com.openexchange.smtp.config.ISMTPProperties;
 import com.openexchange.smtp.config.MailAccountSMTPProperties;
 import com.openexchange.smtp.config.SMTPConfig;
 import com.openexchange.smtp.config.SMTPSessionProperties;
-import com.openexchange.smtp.dataobjects.SMTPMailMessage;
 import com.openexchange.smtp.filler.SMTPMessageFiller;
 import com.openexchange.smtp.services.SMTPServiceRegistry;
 import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
-import com.openexchange.tools.stream.UnsynchronizedByteArrayInputStream;
 import com.sun.mail.smtp.SMTPMessage;
 
 /**
  * {@link SMTPTransport} - The SMTP mail transport.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class SMTPTransport extends MailTransport {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(SMTPTransport.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(SMTPTransport.class));
+
+    private static final class SaslSmtpLoginAction implements PrivilegedExceptionAction<Object> {
+
+        private final Transport transport;
+        private final String server;
+        private final int port;
+        private final String login;
+        private final String pw;
+
+        protected SaslSmtpLoginAction(final Transport transport, final String server, final int port, final String login, final String pw) {
+            super();
+            this.transport = transport;
+            this.server = server;
+            this.port = port;
+            this.login = login;
+            this.pw = pw;
+        }
+
+        public Object run() throws MessagingException {
+            transport.connect(server, port, login, pw);
+            return null;
+        }
+
+    }
 
     private static final String CHARENC_ISO_8859_1 = "ISO-8859-1";
 
+    private static final String KERBEROS_SESSION_SUBJECT = "kerberosSubject";
+
+    private static volatile String staticHostName;
+
+    private static volatile UnknownHostException warnSpam;
+
+    static {
+        try {
+            staticHostName = InetAddress.getLocalHost().getCanonicalHostName();
+        } catch (final UnknownHostException e) {
+            staticHostName = "localhost";
+            warnSpam = e;
+        }
+    }
+
     private final Queue<Runnable> pendingInvocations;
 
     private volatile javax.mail.Session smtpSession;
@@ -129,6 +179,8 @@ public final class SMTPTransport extends MailTransport {
 
     private final Session session;
 
+    private transient Subject kerberosSubject;
+
     private final Context ctx;
 
     private final UserSettingMail usm;
@@ -147,7 +199,7 @@ public final class SMTPTransport extends MailTransport {
 
     /**
      * Constructor
-     * 
+     *
      * @param session The session
      * @throws MailException If initialization fails
      */
@@ -157,7 +209,7 @@ public final class SMTPTransport extends MailTransport {
 
     /**
      * Constructor
-     * 
+     *
      * @param session The session
      * @param accountId The account ID
      * @throws MailException If initialization fails
@@ -206,13 +258,42 @@ public final class SMTPTransport extends MailTransport {
 
     /**
      * Executes the given task. This method returns as soon as the task is scheduled, without waiting for it to be executed.
-     * 
+     *
      * @param task The task to be executed.
      */
     private void invokeLater(final Runnable task) {
         pendingInvocations.offer(task);
     }
 
+    /**
+     * Checks if Kerberos authentication is supposed to be performed.
+     * 
+     * @return <code>true</code> for Kerberos authentication; otherwise <code>false</code>
+     */
+    private boolean isKerberosAuth() {
+        return MailAccount.DEFAULT_ID == accountId && null != kerberosSubject;
+    }
+
+    private static void handlePrivilegedActionException(final PrivilegedActionException e) throws MessagingException, MailException {
+        if (null == e) {
+            return;
+        }
+        final Exception cause = e.getException();
+        if (null == cause) {
+            throw new MailException(MailException.Code.UNEXPECTED_ERROR, e, e.getCause());
+        }
+        if (cause instanceof MessagingException) {
+            throw (MessagingException) cause;
+        }
+        if (cause instanceof MailException) {
+            throw (MailException) cause;
+        }
+        if (cause instanceof AbstractOXException) {
+            throw new MailException((AbstractOXException) cause);
+        }
+        throw new MailException(MailException.Code.UNEXPECTED_ERROR, cause, cause.getCause());
+    }
+
     private javax.mail.Session getSMTPSession() throws MailException {
         if (null == smtpSession) {
             synchronized (this) {
@@ -223,8 +304,25 @@ public final class SMTPTransport extends MailTransport {
                      * Set properties
                      */
                     final ISMTPProperties smtpProperties = smtpConfig.getSMTPProperties();
-                    if (smtpProperties.getSmtpLocalhost() != null) {
-                        smtpProps.put("mail.smtp.localhost", smtpProperties.getSmtpLocalhost());
+                    /*
+                     * Check for Kerberos subject
+                     */
+                    this.kerberosSubject = (Subject) session.getParameter(KERBEROS_SESSION_SUBJECT);
+                    final boolean kerberosAuth = isKerberosAuth();
+                    if (kerberosAuth) {
+                        smtpProps.put("mail.smtp.auth", "true"); 
+                        smtpProps.put("mail.smtp.sasl.enable", "true");
+                        smtpProps.put("mail.smtp.sasl.authorizationid", smtpConfig.getLogin());
+                        smtpProps.put("mail.smtp.sasl.mechanisms", (kerberosAuth ? "GSSAPI" : "PLAIN"));
+                    } else {
+                        smtpProps.put("mail.smtp.auth", smtpProperties.isSmtpAuth() ? "true" : "false");
+                    }
+                    /*
+                     * Localhost, & timeouts
+                     */
+                    final String smtpLocalhost = smtpProperties.getSmtpLocalhost();
+                    if (smtpLocalhost != null) {
+                        smtpProps.put("mail.smtp.localhost", smtpLocalhost);
                     }
                     if (smtpProperties.getSmtpTimeout() > 0) {
                         smtpProps.put("mail.smtp.timeout", String.valueOf(smtpProperties.getSmtpTimeout()));
@@ -232,7 +330,6 @@ public final class SMTPTransport extends MailTransport {
                     if (smtpProperties.getSmtpConnectionTimeout() > 0) {
                         smtpProps.put("mail.smtp.connectiontimeout", String.valueOf(smtpProperties.getSmtpConnectionTimeout()));
                     }
-                    smtpProps.put("mail.smtp.auth", smtpProperties.isSmtpAuth() ? "true" : "false");
                     /*
                      * Check if a secure SMTP connection should be established
                      */
@@ -265,7 +362,27 @@ public final class SMTPTransport extends MailTransport {
                          * Enables the use of the STARTTLS command (if supported by the server) to switch the connection to a TLS-protected
                          * connection before issuing any login commands.
                          */
-                        smtpProps.put("mail.smtp.starttls.enable", "true");
+                        String hostName = smtpLocalhost;
+                        if (null == hostName) {
+                            final HostnameService hostnameService = SMTPServiceRegistry.getServiceRegistry().getService(HostnameService.class);
+                            if (null == hostnameService) {
+                                hostName = getHostName();
+                            } else {
+                                hostName = hostnameService.getHostname(session.getUserId(), session.getContextId());
+                            }
+                            if (null == hostName) {
+                                hostName = getHostName();
+                            }
+                        }
+                        try {
+                            final InetSocketAddress address = new InetSocketAddress(IDNA.toASCII(smtpConfig.getServer()), smtpConfig.getPort());
+                            final Map<String, String> capabilities = SMTPCapabilityCache.getCapabilities(address, smtpConfig.isSecure(), smtpProperties, hostName);
+                            if (capabilities.containsKey("STARTTLS")) {
+                                smtpProps.put("mail.smtp.starttls.enable", "true");
+                            }
+                        } catch (final IOException e) {
+                            smtpProps.put("mail.smtp.starttls.enable", "true");
+                        }
                         /*
                          * Specify the javax.net.ssl.SSLSocketFactory class, this class will be used to create SMTP SSL sockets if TLS
                          * handshake says so.
@@ -326,8 +443,8 @@ public final class SMTPTransport extends MailTransport {
 
     @Override
     public void sendReceiptAck(final MailMessage srcMail, final String fromAddr) throws MailException {
+        SMTPConfig smtpConfig = null;
         try {
-            clearUp();
             final InternetAddress dispNotification = srcMail.getDispositionNotification();
             if (dispNotification == null) {
                 throw new SMTPException(
@@ -353,7 +470,9 @@ public final class SMTPTransport extends MailTransport {
             /*
              * Set to
              */
-            smtpMessage.addRecipients(RecipientType.TO, new Address[] { dispNotification });
+            final Address[] recipients = new Address[] { dispNotification };
+            checkRecipients(recipients);
+            smtpMessage.addRecipients(RecipientType.TO, recipients);
             /*
              * Set header
              */
@@ -363,7 +482,7 @@ public final class SMTPTransport extends MailTransport {
              * Subject
              */
             final Locale locale = UserStorage.getStorageUser(session.getUserId(), ctx).getLocale();
-            final StringHelper strHelper = new StringHelper(locale);
+            final StringHelper strHelper = StringHelper.valueOf(locale);
             smtpMessage.setSubject(strHelper.getString(MailStrings.ACK_SUBJECT));
             /*
              * Sent date in UTC time
@@ -377,7 +496,7 @@ public final class SMTPTransport extends MailTransport {
             /*
              * Set common headers
              */
-            final SMTPConfig smtpConfig = getTransportConfig0();
+            smtpConfig = getTransportConfig0();
             new SMTPMessageFiller(smtpConfig.getSMTPProperties(), session, ctx, usm).setCommonHeaders(smtpMessage);
             /*
              * Compose body
@@ -429,14 +548,25 @@ public final class SMTPTransport extends MailTransport {
             final long start = System.currentTimeMillis();
             final Transport transport = getSMTPSession().getTransport(SMTPProvider.PROTOCOL_SMTP.getName());
             try {
+                final String server = IDNA.toASCII(smtpConfig.getServer());
+                final int port = smtpConfig.getPort();
                 if (smtpConfig.getSMTPProperties().isSmtpAuth()) {
-                    transport.connect(
-                        smtpConfig.getServer(),
-                        smtpConfig.getPort(),
-                        smtpConfig.getLogin(),
-                        encodePassword(smtpConfig.getPassword()));
+                    if (isKerberosAuth()) {
+                        try {
+                            Subject.doAs(kerberosSubject, new SaslSmtpLoginAction(
+                                transport,
+                                server,
+                                port,
+                                smtpConfig.getLogin(),
+                                encodePassword(smtpConfig.getPassword())));
+                        } catch (final PrivilegedActionException e) {
+                            handlePrivilegedActionException(e);
+                        }
+                    } else {
+                        transport.connect(server, port, smtpConfig.getLogin(), encodePassword(smtpConfig.getPassword()));
+                    }
                 } else {
-                    transport.connect();
+                    transport.connect(server, port, null, null);
                 }
             } catch (final javax.mail.AuthenticationFailedException e) {
                 throw new MIMEMailException(MIMEMailException.Code.TRANSPORT_INVALID_CREDENTIALS, e, smtpConfig.getServer(), e.getMessage());
@@ -449,32 +579,44 @@ public final class SMTPTransport extends MailTransport {
                 transport.close();
             }
         } catch (final MessagingException e) {
-            throw MIMEMailException.handleMessagingException(e);
+            throw MIMEMailException.handleMessagingException(e, smtpConfig);
         }
     }
 
     @Override
     public MailMessage sendRawMessage(final byte[] asciiBytes, final Address[] allRecipients) throws MailException {
+        final SMTPConfig smtpConfig = getTransportConfig0();
         try {
-            clearUp();
-            final SMTPMessage smtpMessage = new SMTPMessage(getSMTPSession(), new UnsynchronizedByteArrayInputStream(asciiBytes));
+            final SMTPMessage smtpMessage = new SMTPMessage(getSMTPSession(), MimeHeaderNameChecker.sanitizeHeaderNames(asciiBytes));
+            smtpMessage.removeHeader("x-original-headers");
             /*
              * Check recipients
              */
             final Address[] recipients = allRecipients == null ? smtpMessage.getAllRecipients() : allRecipients;
-            if ((recipients == null) || (recipients.length == 0)) {
-                throw new SMTPException(SMTPException.Code.MISSING_RECIPIENTS);
-            }
+            checkRecipients(recipients);
             try {
                 final long start = System.currentTimeMillis();
                 final Transport transport = getSMTPSession().getTransport(SMTPProvider.PROTOCOL_SMTP.getName());
-                final SMTPConfig smtpConfig = getTransportConfig0();
                 try {
+                    final String server = IDNA.toASCII(smtpConfig.getServer());
+                    final int port = smtpConfig.getPort();
                     if (smtpConfig.getSMTPProperties().isSmtpAuth()) {
-                        final String encPass = encodePassword(smtpConfig.getPassword());
-                        transport.connect(smtpConfig.getServer(), smtpConfig.getPort(), smtpConfig.getLogin(), encPass);
+                        if (isKerberosAuth()) {
+                            try {
+                                Subject.doAs(kerberosSubject, new SaslSmtpLoginAction(
+                                    transport,
+                                    server,
+                                    port,
+                                    smtpConfig.getLogin(),
+                                    encodePassword(smtpConfig.getPassword())));
+                            } catch (final PrivilegedActionException e) {
+                                handlePrivilegedActionException(e);
+                            }
+                        } else {
+                            transport.connect(server, port, smtpConfig.getLogin(), encodePassword(smtpConfig.getPassword()));
+                        }
                     } else {
-                        transport.connect();
+                        transport.connect(server, port, null, null);
                     }
                 } catch (final javax.mail.AuthenticationFailedException e) {
                     throw new MIMEMailException(
@@ -491,28 +633,27 @@ public final class SMTPTransport extends MailTransport {
                     transport.close();
                 }
             } catch (final MessagingException e) {
-                throw MIMEMailException.handleMessagingException(e);
+                throw MIMEMailException.handleMessagingException(e, smtpConfig);
             }
             return MIMEMessageConverter.convertMessage(smtpMessage);
         } catch (final MessagingException e) {
-            throw MIMEMailException.handleMessagingException(e);
+            throw MIMEMailException.handleMessagingException(e, smtpConfig);
         }
     }
 
     @Override
     public MailMessage sendMailMessage(final ComposedMailMessage composedMail, final ComposeType sendType, final Address[] allRecipients) throws MailException {
+        final SMTPConfig smtpConfig = getTransportConfig0();
         try {
-            clearUp();
             final SMTPMessage smtpMessage = new SMTPMessage(getSMTPSession());
             /*
              * Fill message dependent on send type
              */
             final long startPrep = System.currentTimeMillis();
-            final SMTPConfig smtpConfig = getTransportConfig0();
             final SMTPMessageFiller smtpFiller = new SMTPMessageFiller(smtpConfig.getSMTPProperties(), session, ctx, usm);
             composedMail.setFiller(smtpFiller);
             try {
-                smtpFiller.fillMail((SMTPMailMessage) composedMail, smtpMessage, sendType);
+                smtpFiller.fillMail(composedMail, smtpMessage, sendType);
 
                 /*
                  * Check recipients
@@ -527,23 +668,38 @@ public final class SMTPTransport extends MailTransport {
                 } else {
                     recipients = allRecipients;
                 }
-
-                if ((recipients == null) || (recipients.length == 0)) {
-                    throw new SMTPException(SMTPException.Code.MISSING_RECIPIENTS);
-                }
+                checkRecipients(recipients);
                 smtpFiller.setSendHeaders(composedMail, smtpMessage);
+                /*
+                 * Drop special "x-original-headers" header
+                 */
+                smtpMessage.removeHeader("x-original-headers");
                 if (LOG.isDebugEnabled()) {
-                    LOG.debug(new StringBuilder(128).append("SMTP mail prepared for transport in ").append(
-                        System.currentTimeMillis() - startPrep).append("msec").toString());
+                    LOG.debug(new StringBuilder(128).append("SMTP mail prepared for transport in ").append(System.currentTimeMillis()
+                        - startPrep).append("msec").toString());
                 }
                 final long start = System.currentTimeMillis();
                 final Transport transport = getSMTPSession().getTransport(SMTPProvider.PROTOCOL_SMTP.getName());
                 try {
+                    final String server = IDNA.toASCII(smtpConfig.getServer());
+                    final int port = smtpConfig.getPort();
                     if (smtpConfig.getSMTPProperties().isSmtpAuth()) {
-                        final String encPass = encodePassword(smtpConfig.getPassword());
-                        transport.connect(smtpConfig.getServer(), smtpConfig.getPort(), smtpConfig.getLogin(), encPass);
+                        if (isKerberosAuth()) {
+                            try {
+                                Subject.doAs(kerberosSubject, new SaslSmtpLoginAction(
+                                    transport,
+                                    server,
+                                    port,
+                                    smtpConfig.getLogin(),
+                                    encodePassword(smtpConfig.getPassword())));
+                            } catch (final PrivilegedActionException e) {
+                                handlePrivilegedActionException(e);
+                            }
+                        } else {
+                            transport.connect(server, port, smtpConfig.getLogin(), encodePassword(smtpConfig.getPassword()));
+                        }
                     } else {
-                        transport.connect();
+                        transport.connect(server, port, null, null);
                     }
                 } catch (final javax.mail.AuthenticationFailedException e) {
                     throw new MIMEMailException(
@@ -567,7 +723,7 @@ public final class SMTPTransport extends MailTransport {
             }
             return MIMEMessageConverter.convertMessage(smtpMessage);
         } catch (final MessagingException e) {
-            throw MIMEMailException.handleMessagingException(e);
+            throw MIMEMailException.handleMessagingException(e, smtpConfig);
         } catch (final IOException e) {
             throw new SMTPException(SMTPException.Code.IO_ERROR, e, e.getMessage());
         }
@@ -589,12 +745,29 @@ public final class SMTPTransport extends MailTransport {
     @Override
     protected void shutdown() throws MailException {
         SMTPSessionProperties.resetDefaultSessionProperties();
+        SMTPCapabilityCache.tearDown();
     }
 
     @Override
     protected void startup() throws MailException {
-        if (LOG.isTraceEnabled()) {
-            LOG.trace("SMTPTransport.startup()");
+        SMTPCapabilityCache.init();
+    }
+
+    private static void checkRecipients(final Address[] recipients) throws SMTPException {
+        if ((recipients == null) || (recipients.length == 0)) {
+            throw new SMTPException(SMTPException.Code.MISSING_RECIPIENTS);
+        }
+        final ConfigurationService service = SMTPServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+        if (null != service) {
+            final Filter filter = service.getFilterFromProperty("com.openexchange.mail.transport.redirectWhitelist");
+            if (null != filter) {
+                for (final Address address : recipients) {
+                    final InternetAddress internetAddress = (InternetAddress) address;
+                    if (!filter.accepts(internetAddress.getAddress())) {
+                        throw new SMTPException(SMTPException.Code.RECIPIENT_NOT_ALLOWED, internetAddress.toUnicodeString());
+                    }
+                }
+            }
         }
     }
 
@@ -623,21 +796,35 @@ public final class SMTPTransport extends MailTransport {
             throw MIMEMailException.handleMessagingException(e);
         }
         boolean close = false;
+        final SMTPConfig config = getTransportConfig0();
         try {
-            final SMTPConfig config = getTransportConfig0();
             try {
+                final String server = IDNA.toASCII(config.getServer());
+                final int port = config.getPort();
                 if (config.getSMTPProperties().isSmtpAuth()) {
-                    final String encPass = encodePassword(config.getPassword());
-                    transport.connect(config.getServer(), config.getPort(), config.getLogin(), encPass);
+                    if (isKerberosAuth()) {
+                        try {
+                            Subject.doAs(kerberosSubject, new SaslSmtpLoginAction(
+                                transport,
+                                server,
+                                port,
+                                config.getLogin(),
+                                encodePassword(config.getPassword())));
+                        } catch (final PrivilegedActionException e) {
+                            handlePrivilegedActionException(e);
+                        }
+                    } else {
+                        transport.connect(server, port, config.getLogin(), encodePassword(config.getPassword()));
+                    }
                 } else {
-                    transport.connect();
+                    transport.connect(server, port, null, null);
                 }
                 close = true;
             } catch (final javax.mail.AuthenticationFailedException e) {
                 throw new MIMEMailException(MIMEMailException.Code.TRANSPORT_INVALID_CREDENTIALS, e, config.getServer(), e.getMessage());
             }
         } catch (final MessagingException e) {
-            throw MIMEMailException.handleMessagingException(e);
+            throw MIMEMailException.handleMessagingException(e, config);
         } finally {
             if (close) {
                 try {
@@ -662,79 +849,61 @@ public final class SMTPTransport extends MailTransport {
         }
     }
 
-    private static void saveChangesSafe(final SMTPMessage smtpMessage) throws MailException {
+    private void saveChangesSafe(final SMTPMessage smtpMessage) throws MailException {
         try {
-            try {
-                smtpMessage.saveChanges();
-            } catch (final javax.mail.internet.ParseException e) {
-                /*-
-                 * Probably parsing of a Content-Type header failed.
-                 * 
-                 * Try to sanitize parameter list headers
-                 */
-                sanitizeContentTypeHeaders(smtpMessage, new ContentType());
+            saveChanges(smtpMessage);
+            /*
+             * Change Message-Id header appropriately
+             */
+            final String messageId = smtpMessage.getHeader("Message-ID", null);
+            if (null != messageId) {
                 /*
-                 * ... and retry
+                 * Somewhat of: <744810669.1.1314981157714.JavaMail.username@host.com>
                  */
-                smtpMessage.saveChanges();
+                final HostnameService hostnameService = SMTPServiceRegistry.getServiceRegistry().getService(HostnameService.class);
+                String hostName;
+                if (null == hostnameService) {
+                    hostName = getHostName();
+                } else {
+                    hostName = hostnameService.getHostname(session.getUserId(), session.getContextId());
+                }
+                if (null == hostName) {
+                    hostName = getHostName();
+                }
+                final int pos = messageId.indexOf('@');
+                if (pos > 0 ) {
+                    final StringBuilder mid = new StringBuilder(messageId.substring(0, pos + 1)).append(hostName);
+                    if (messageId.charAt(0) == '<') {
+                        mid.append('>');
+                    }
+                    smtpMessage.setHeader("Message-ID", mid.toString());
+                } else {
+                    smtpMessage.setHeader("Message-ID", messageId + hostName);
+                }
             }
         } catch (final MessagingException e) {
             throw MIMEMailException.handleMessagingException(e);
         }
     }
 
-    private static void sanitizeContentTypeHeaders(final Part part, final ContentType sanitizer) throws MailException {
-        final DataHandler dh;
-        try {
-            dh = part.getDataHandler();
-        } catch (final MessagingException e) {
-            throw MIMEMailException.handleMessagingException(e);
+    private static String getHostName() {
+        final Map<String, Object> logProperties = LogProperties.optLogProperties();
+        if (null == logProperties) {
+            return getStaticHostName();
         }
-        if (dh == null) {
-            return;
+        final String serverName = (String) logProperties.get("com.openexchange.ajp13.serverName");
+        if (null == serverName) {
+            return getStaticHostName();
         }
-        try {
-            final String type = dh.getContentType();
-            sanitizer.setContentType(type);
-            try {
-                /*
-                 * Try to parse with JavaMail Content-Type implementation
-                 */
-                new javax.mail.internet.ContentType(type);
-            } catch (final javax.mail.internet.ParseException e) {
-                /*
-                 * Sanitize Content-Type header
-                 */
-                final String cts = sanitizer.toString(true);
-                try {
-                    new javax.mail.internet.ContentType(cts);
-                } catch (final javax.mail.internet.ParseException pe) {
-                    /*
-                     * Still not parseable
-                     */
-                    throw new MailException(MailException.Code.INVALID_CONTENT_TYPE, e, type);
-                }
-                part.setDataHandler(new DataHandlerWrapper(dh, cts));
-                part.setHeader("Content-Type", cts);
-            }
-            /*
-             * Check for recursive invocation
-             */
-            if (sanitizer.startsWith("multipart/")) {
-                final Object o = dh.getContent();
-                if (o instanceof MimeMultipart) {
-                    final MimeMultipart mm = (MimeMultipart) o;
-                    final int count = mm.getCount();
-                    for (int i = 0; i < count; i++) {
-                        sanitizeContentTypeHeaders(mm.getBodyPart(i), sanitizer);
-                    }
-                }
-            }
-        } catch (final MessagingException e) {
-            throw MIMEMailException.handleMessagingException(e);
-        } catch (final IOException e) {
-            throw new MailException(MailException.Code.IO_ERROR, e, e.getMessage());
+        return serverName;
+    }
+
+    private static String getStaticHostName() {
+        final UnknownHostException warning = warnSpam;
+        if (warning != null) {
+            LOG.error("Can't resolve my own hostname, using 'localhost' instead, which is certainly not what you want!", warning);
         }
+        return staticHostName;
     }
 
 }
