@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,18 +57,18 @@ import com.openexchange.mailaccount.MailAccount;
 
 /**
  * {@link MailAccountSMTPProperties} - SMTP properties read from mail account with fallback to properties read from properties file.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class MailAccountSMTPProperties extends MailAccountTransportProperties implements ISMTPProperties {
 
-    private static final Log LOG = LogFactory.getLog(MailAccountSMTPProperties.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(MailAccountSMTPProperties.class));
 
     private final MailAccount mailAccount;
 
     /**
      * Initializes a new {@link MailAccountSMTPProperties}.
-     * 
+     *
      * @param mailAccount The mail account providing the properties
      * @throws IllegalArgumentException If provided mail account is <code>null</code>
      */
