@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,9 +55,9 @@ import com.openexchange.session.Session;
 
 /**
  * {@link SMTPDocumentPart}
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
- * 
+ *
  */
 public final class SMTPDocumentPart extends InfostoreDocumentMailPart {
 
@@ -65,7 +65,7 @@ public final class SMTPDocumentPart extends InfostoreDocumentMailPart {
 
 	/**
 	 * Constructor
-	 * 
+	 *
 	 * @param documentId
 	 *            The document's unique ID
 	 * @param session
@@ -73,7 +73,7 @@ public final class SMTPDocumentPart extends InfostoreDocumentMailPart {
 	 * @throws MailException
 	 *             If infostore document cannot be read
 	 */
-	public SMTPDocumentPart(final int documentId, final Session session) throws MailException {
+	public SMTPDocumentPart(final String documentId, final Session session) throws MailException {
 		super(documentId, session);
 	}
 
