@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,7 +49,7 @@
 
 package com.openexchange.smtp.dataobjects;
 
-import com.openexchange.groupware.upload.impl.UploadFile;
+import com.openexchange.groupware.upload.UploadFile;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.dataobjects.MailPart;
 import com.openexchange.mail.dataobjects.compose.UploadFileMailPart;
@@ -58,9 +58,9 @@ import com.openexchange.mail.dataobjects.compose.UploadFileMailPart;
  * {@link SMTPFilePart} - A {@link MailPart} implementation that keeps a
  * reference to a temporary uploaded file that shall be added as an attachment
  * later
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
- * 
+ *
  */
 public final class SMTPFilePart extends UploadFileMailPart {
 
@@ -68,7 +68,7 @@ public final class SMTPFilePart extends UploadFileMailPart {
 
 	/**
 	 * Constructor
-	 * 
+	 *
 	 * @throws MailException
 	 *             If upload file's content type cannot be parsed
 	 */
