@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -62,15 +62,15 @@ import com.openexchange.session.Session;
 /**
  * {@link SMTPMailMessage} - Extends the {@link ComposedMailMessage} class to
  * compose SMTP messages.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
- * 
+ *
  */
 public final class SMTPMailMessage extends ComposedMailMessage {
 
 	private static final long serialVersionUID = 9031153888416594901L;
 
-	private SMTPBodyPart mailPart;
+	private TextBodyMailPart mailPart;
 
 	private String uid;
 
@@ -83,7 +83,7 @@ public final class SMTPMailMessage extends ComposedMailMessage {
 	 * is defined through {@link #setContentType(String)} or
 	 * {@link #setContentType(com.openexchange.mail.mime.ContentType)}. The HTML
 	 * is then converted appropriately.
-	 * 
+	 *
 	 * @param htmlMailBody
 	 *            The mail body as HTML content
 	 * @param session
@@ -93,13 +93,13 @@ public final class SMTPMailMessage extends ComposedMailMessage {
 	 */
 	public SMTPMailMessage(final String htmlMailBody, final Session session, final Context ctx) {
 		super(session, ctx);
-		this.mailPart = new SMTPBodyPart(htmlMailBody);
+		mailPart = new SMTPBodyPart(htmlMailBody);
 		enclosedParts = new ArrayList<MailPart>();
 	}
 
 	/**
 	 * Initializes a new {@link SMTPMailMessage}
-	 * 
+	 *
 	 * @param session
 	 *            The session providing needed user data
 	 * @param ctx
@@ -117,7 +117,12 @@ public final class SMTPMailMessage extends ComposedMailMessage {
 
 	@Override
 	public void setBodyPart(final TextBodyMailPart mailPart) {
-		this.mailPart = (SMTPBodyPart) mailPart;
+		this.mailPart = mailPart;
+	}
+
+	@Override
+	public TextBodyMailPart getBodyPart() {
+	    return mailPart;
 	}
 
 	@Override
@@ -154,7 +159,7 @@ public final class SMTPMailMessage extends ComposedMailMessage {
 	 * Removes the enclosed part at the specified position. Shifts any
 	 * subsequent parts to the left (subtracts one from their indices). Returns
 	 * the part that was removed.
-	 * 
+	 *
 	 * @param index
 	 *            The index position
 	 * @return The removed part
@@ -166,7 +171,7 @@ public final class SMTPMailMessage extends ComposedMailMessage {
 
 	/**
 	 * Adds an instance of {@link MailPart} to enclosed parts
-	 * 
+	 *
 	 * @param part
 	 *            The instance of {@link MailPart} to add
 	 */
@@ -182,7 +187,7 @@ public final class SMTPMailMessage extends ComposedMailMessage {
 
 	@Override
 	public void setMailId(final String id) {
-		this.uid = id;
+		uid = id;
 	}
 
 }
