@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,26 +54,31 @@ import java.util.Dictionary;
 import java.util.Hashtable;
 import org.osgi.framework.BundleActivator;
 import org.osgi.framework.ServiceRegistration;
+import org.osgi.util.tracker.ServiceTracker;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.mail.transport.TransportProvider;
 import com.openexchange.mailaccount.MailAccountStorageService;
 import com.openexchange.server.osgiservice.DeferredActivator;
+import com.openexchange.server.osgiservice.RegistryCustomizer;
 import com.openexchange.server.osgiservice.ServiceRegistry;
 import com.openexchange.smtp.SMTPProvider;
 
 /**
  * {@link SMTPActivator} - The {@link BundleActivator activator} for SMTP bundle.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class SMTPActivator extends DeferredActivator {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(SMTPActivator.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(SMTPActivator.class));
 
     private final Dictionary<String, String> dictionary;
 
     private ServiceRegistration smtpServiceRegistration;
 
+    private ServiceTracker tracker;
+
     /**
      * Initializes a new {@link SMTPActivator}
      */
@@ -117,13 +122,15 @@ public final class SMTPActivator extends DeferredActivator {
                 final ServiceRegistry registry = getServiceRegistry();
                 registry.clearRegistry();
                 final Class<?>[] classes = getNeededServices();
-                for (int i = 0; i < classes.length; i++) {
-                    final Object service = getService(classes[i]);
+                for (final Class<?> classe : classes) {
+                    final Object service = getService(classe);
                     if (null != service) {
-                        registry.addService(classes[i], service);
+                        registry.addService(classe, service);
                     }
                 }
             }
+            tracker = new ServiceTracker(context, HostnameService.class.getName(), new RegistryCustomizer<HostnameService>(context, HostnameService.class, getServiceRegistry()));
+            tracker.open();
             smtpServiceRegistration = context.registerService(TransportProvider.class.getName(), SMTPProvider.getInstance(), dictionary);
         } catch (final Throwable t) {
             LOG.error(t.getMessage(), t);
@@ -139,6 +146,10 @@ public final class SMTPActivator extends DeferredActivator {
                 smtpServiceRegistration.unregister();
                 smtpServiceRegistration = null;
             }
+            if (null != tracker) {
+                tracker.close();
+                tracker = null;
+            }
             /*
              * Clear service registry
              */
